#	
#	Copyright 2009 Amr Hassan <amr.hassan@gmail.com>
#	
#	This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#	MA 02110-1301, USA.

from __base__ import *

class Player(PlayerBase):
	
	def __init__(self):
		PlayerBase.__init__(self, "Banshee", "1.5", "org.bansheeproject.Banshee")
	
	def is_playing(self):
		try:
			obj = self._get_dbus_object("/org/bansheeproject/Banshee/PlayerEngine")		
			return str(obj.GetCurrentState()) == "playing"
		except:
			return
	
	def get_track(self):
		if not self.is_playing(): return
		
		try:
			obj = self._get_dbus_object("/org/bansheeproject/Banshee/PlayerEngine")
			data = obj.GetCurrentTrack()
		except:
			return
		
		if not "artist" in data.keys() or not "name" in data.keys() or not "album" in data.keys():
			return
		
		return {"artist": unicode(data["artist"]), "title": unicode(data["name"]), "album": unicode(data["album"]), 
				"uri": unicode(data["URI"])}
	
	def play_pause(self):
		try:
			obj = self._get_dbus_object("/org/bansheeproject/Banshee/PlayerEngine")
			obj.TogglePlaying()
		except:
			return
			
	def get_volume(self):
		try:
			obj = self._get_dbus_object("/org/bansheeproject/Banshee/PlayerEngine")
			return float(obj.GetVolume())
		except:
			return
	
	def set_volume(self, value):
		try:
			obj = self._get_dbus_object("/org/bansheeproject/Banshee/PlayerEngine")
			obj.SetVolume(dbus.UInt16(value))
		except:
			return
	
	def next(self):
		try:
			obj = self._get_dbus_object("/org/bansheeproject/Banshee/PlaybackController")
			obj.Next(False)
		except:
			return
	
	def previous(self):
		try:
			obj = self._get_dbus_object("/org/bansheeproject/Banshee/PlaybackController")		
			obj.Previous(False)
		except:
			return



#	
#	Copyright 2009 Amr Hassan <amr.hassan@gmail.com>
#	
#	This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#	MA 02110-1301, USA.

from __base__ import *

class Player(PlayerBase):
	
	def __init__(self):
		PlayerBase.__init__(self, "Exaile", "0.2", "org.exaile.DBusInterface")
	
	def get_track(self):
		if not self.is_playing(): return
		
		try:
			obj = self._get_dbus_object("/DBusInterfaceObject")
		except:
			return
		
		return {"artist": unicode(obj.get_artist()),
				"title": unicode(obj.get_title()),
				"album": unicode(obj.get_album()),
				"uri": u""
				}
	
	def is_playing(self):
		try:
			obj = self._get_dbus_object("/DBusInterfaceObject")
		except:
			return
		
		return obj.query().startswith("status: playing")
	
	def play_pause(self):
		try:
			obj = self._get_dbus_object("/DBusInterfaceObject")
		except:
			return
		
		obj.play_pause()
	
	def next(self):
		try:
			obj = self._get_dbus_object("/DBusInterfaceObject")
		except:
			return
		
		obj.next_track()
	
	def previous(self):
		try:
			obj = self._get_dbus_object("/DBusInterfaceObject")
		except:
			return
		
		obj.prev_track()
	
	def get_volume(self):
		try:
			obj = self._get_dbus_object("/DBusInterfaceObject")
		except:
			return
		
		return float(obj.get_volume())
	
	def set_volume(self, value):
		try:
			obj = self._get_dbus_object("/DBusInterfaceObject")
		except:
			return
		
		current = self.get_volume()
		
		value = float(value)
		
		if value < current:
			obj.decrease_volume(current - value)
		else:
			obj.increase_volume(value - current)
	



#	
#	Copyright 2009 Amr Hassan <amr.hassan@gmail.com>
#	
#	This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#	MA 02110-1301, USA.

import dbus
import imp
import glob
import os
import sys

import nowplaying.players

def _get_players():
	players_path = nowplaying.players.__path__[0]
	
	players = []
	
	for path in glob.glob(players_path + "/*.py"):
		
		name = os.path.basename(path)[:-3]
		if name.startswith("__"): continue
		
		full_name = "%s.players.%s" %(__package__, name)
		
		if not full_name in sys.modules.keys():
			imp.load_module(full_name, *imp.find_module(name, [players_path]))
		
		players.append(sys.modules[full_name].Player())
	
	return players

_players = _get_players()

def get_running_players():
	running = []
	for p in _players:
		if p.is_running():
			running.append(p)
		
	return running

def get_player(player_id):
	"""Returns a player object if the player's running. None otherwise."""
	
	for player in get_running_players():
		if player.get_id() == player_id:
			return player



#	
#	Copyright 2009 Amr Hassan <amr.hassan@gmail.com>
#	
#	This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#	MA 02110-1301, USA.

import dbus

class PlayerBase(object):
	"""A base for player objects"""
	
	def __init__(self, name, version, dbus_name):
		self.name = name
		self.version = version
		self.dbus_name = dbus_name
		
		self.bus = dbus.SessionBus()
	
	def _get_dbus_object(self, path):
		return self.bus.get_object(self.dbus_name, path)
		
	def __repr__(self):
		return self.get_name()
	
	def get_track(self):
		return NotImplemented
	
	def get_name(self):
		return self.name
	
	def is_running(self):
		return self.dbus_name in self.bus.list_activatable_names()
	
	def is_playing(self):
		return NotImplemented
	
	def play_payse(self):
		pass
	
	def next(self):
		pass
	
	def previous(self):
		pass
	
	def get_volume(self):
		return NotImplemented
	
	def set_volume(self, value):
		pass
	
	def get_id(self):
		return "%s-%s" %(self.name, unicode(self.version))
		
	def get_snapshot(self):
		"""Returns a snapshots of all the values of this player."""
		
		return {"volume": self.get_volume(),
				"track": self.get_track(),
				"name": self.name,
				"version": self.version,
				"is_playing": self.is_playing(),
				"player_id": self.get_id(),
				}



#	
#	Copyright 2009 Amr Hassan <amr.hassan@gmail.com>
#	
#	This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#	MA 02110-1301, USA.

from __base__ import *

class Player(PlayerBase):
	
	def __init__(self):
		PlayerBase.__init__(self, "Audacious", "1.5", "org.mpris.audacious")
		
	def get_track(self):
		try:
			object = self._get_dbus_object("/Player")
			metadata = object.GetMetadata()
		except:
			return
		
		if "artist" in metadata and "title" in metadata and "album" in metadata:
			return {"artist": unicode(metadata["artist"]), "title": unicode(metadata["title"]), "album": unicode(metadata["album"]), "uri": unicode(metadata["uri"])}
	
	def is_playing(self):
		try:
			object = self._get_dbus_object("/org/atheme/audacious")
			return str(object.Status()) == "playing"
		except:
			return
	
	def play_pause(self):
		try:
			object = self._get_dbus_object("/org/atheme/audacious")
			object.PlayPause()
		except:
			pass
	
	def next(self):
		try:
			object = self._get_dbus_object("/Player")
			object.Next()
		except:
			pass
	
	def previous(self):		
		try:
			object = self._get_dbus_object("/Player")
			object.Prev()
		except:
			return
	
	def get_volume(self):
		try:
			object = self._get_dbus_object("/org/atheme/audacious")
			return float(object.Volume()[0])
		except:
			return
	
	def set_volume(self, value):
		try:
			object = self._get_dbus_object("/org/atheme/audacious")
			object.SetVolume(value, value)
		except:
			return
